==> comedi-0.6.7.tgz: comedi-0.6.7/lib/sv.c <==
/*
    lib/sv.c
    comedi library routines - sv section

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <comedi.h>
#include <string.h>
#include <comedilib.h>





int comedi_sv_init(comedi_sv_t *it,comedi_t *dev,unsigned int subd,unsigned int chan)
{
	memset(it,0,sizeof(*it));

	it->dev=dev;
	it->subdevice=subd;
	it->chan=chan;
	it->n=100;

	return comedi_sv_update(it);
}

int comedi_sv_update(comedi_sv_t *it)
{
	it->rangetype=comedi_get_rangetype(it->dev,it->subdevice,it->chan);
	it->maxdata=comedi_get_maxdata(it->dev,it->subdevice,it->chan);

	/* check current range policy */
	/* check n */

	if(it->rangetype<0)return -1;

	return 0;
}

int comedi_sv_measure(comedi_sv_t *it,double *data)
{
	comedi_trig t;
	int ret=0;
	sampl_t *val;
	unsigned int chan;
	comedi_range *rng;
	double sum=0;
	int i;

	val=malloc(sizeof(sampl_t)*it->n);

	chan=CR_PACK(it->chan,it->range,it->aref);
	
	t.subdev=it->subdevice;
	t.mode=0;
	t.flags=0;
	t.n_chan=1;
	t.chanlist=&chan;
	t.data=val;
	t.n=it->n;
	t.trigsrc=0;
	t.trigvar=0;
	t.trigvar1=0;

	rng=comedi_get_ranges(it->rangetype);
	if(!rng){
		ret=-1;
		goto out;
	}
	rng+=it->range;

#if 1
	t.n=1;
	for(i=0;i<it->n;i++){
		ret=comedi_trigger(it->dev->fd,&t);
		if(ret<0)goto out;
		sum+=comedi_to_phys(val[0],rng,it->maxdata);
	}
	*data=sum/it->n;
#else
	ret=comedi_trigger(it->dev->fd,&t);
	if(ret<0)goto out;
	for(i=0;i<it->n;i++){
		sum+=comedi_to_phys(val[i],rng,it->maxdata);
	}
#endif
	*data=sum/it->n;

out:
	free(val);

	return ret;
}

==> comedi-0.6.10.tgz: comedi-0.6.10/lib/sv.c <==
/*
    lib/sv.c
    comedi library routines - sv section

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <comedi.h>
#include <string.h>
#include <comedilib.h>





int comedi_sv_init(comedi_sv_t *it,comedi_t *dev,unsigned int subd,unsigned int chan)
{
	memset(it,0,sizeof(*it));

	it->dev=dev;
	it->subdevice=subd;
	it->chan=chan;
	it->n=100;

	return comedi_sv_update(it);
}

int comedi_sv_update(comedi_sv_t *it)
{
	it->rangetype=comedi_get_rangetype(it->dev,it->subdevice,it->chan);
	it->maxdata=comedi_get_maxdata(it->dev,it->subdevice,it->chan);

	/* check current range policy */
	/* check n */

	if(it->rangetype<0)return -1;

	return 0;
}

int comedi_sv_measure(comedi_sv_t *it,double *data)
{
	comedi_trig t;
	int ret=0;
	sampl_t *val;
	unsigned int chan;
	comedi_range *rng;
	double sum;
	int i;
	int n_samples;

	val=malloc(sizeof(sampl_t)*it->n);

	chan=CR_PACK(it->chan,it->range,it->aref);
	
	t.subdev=it->subdevice;
	t.mode=0;
	t.flags=0;
	t.n_chan=1;
	t.chanlist=&chan;
	t.data=val;
	t.n=it->n;
	t.trigsrc=0;
	t.trigvar=0;
	t.trigvar1=0;

	rng=comedi_get_ranges(it->rangetype);
	if(!rng){
		ret=-1;
		goto out;
	}
	rng+=it->range;

	n_samples=t.n;
	while(n_samples>0){
		i=comedi_trigger(it->dev->fd,&t);
		if(i<=0){
			ret=i;
			goto out;
		}
		n_samples-=i;
		t.n-=i;
		t.data+=i;
	}

	sum=0;
	for(i=0;i<it->n;i++){
		sum+=comedi_to_phys(val[i],rng,it->maxdata);
	}
	*data=sum/it->n;

out:
	free(val);

	return ret;
}
